extern crate chrono;
extern crate feed_rs;
extern crate maud;
extern crate reqwest;

use std::error::Error;
use std::fs::write;
use std::fs::DirBuilder;
use std::path::Path;
mod site_generator;
mod utilities;
mod web_fetchers;
use rayon::prelude::*;

fn main() -> Result<(), Box<dyn Error>> {
    simple_logger::init_with_level(log::Level::Info).unwrap();

    let mut featured = utilities::read_feed("featured_feeds.txt");

    featured = featured
        .iter_mut()
        .map(|post| {
            post.score += 60 * 24 * 2;
            post.clone()
        })
        .collect::<Vec<_>>();

    let mut entries = utilities::read_feed("feeds.txt");

    entries.extend(featured);

    entries.retain(|entry| entry.score.is_positive());

    entries.par_iter_mut().for_each(|entry| {
        if entry.image_url.is_some() {
            entry.score += 300;
        } else {
            match web_fetchers::fetch_social_image(entry.link.clone()) {
                Ok(social_image_url) => {
                    entry.image_url = Some(social_image_url);
                }
                Err(error) => {
                    log::info!("{}: {}", error, entry.link.clone());
                    entry.score += -600;
                }
            }
        }
    });

    entries.retain(|entry| entry.score.is_positive());
    entries.sort();

    let index = site_generator::generate_index(entries.clone());
    let index_path = Path::new("output/index.html");
    DirBuilder::new()
        .recursive(true)
        .create(index_path.parent().unwrap())
        .unwrap();

    match write(index_path, index.into_string()) {
        Ok(_) => log::info!("Successfully wrote to {}", index_path.display()),
        Err(e) => log::error!("Failed to write to {}: {}", index_path.display(), e),
    }

    let feed = site_generator::generate_rss(entries.clone());
    let feed_path = Path::new("output/feed.xml");
    DirBuilder::new()
        .recursive(true)
        .create(feed_path.parent().unwrap())
        .unwrap();

    match write(feed_path, feed) {
        Ok(_) => log::info!("Successfully wrote to {}", feed_path.display()),
        Err(e) => log::error!("Failed to write to {}: {}", feed_path.display(), e),
    }

    Ok(())
}
